@@ -30,6 +30,7 @@
 
 #include "seahorse-action.h"
 #include "seahorse-actions.h"
+#include "seahorse-object-list.h"
 #include "seahorse-progress.h"
 #include "seahorse-registry.h"
 #include "seahorse-util.h"
@@ -69,29 +70,29 @@ on_new_keyring (GtkAction *action,
 
 static void
 on_new_item (GtkAction *action,
-                 gpointer unused)
+             gpointer unused)
 {
 	seahorse_gkr_add_item_show (seahorse_action_get_window (action));
 }
 
 static const GtkActionEntry BACKEND_ACTIONS[] = {
-	{ "gkr-keyring-new", NULL, N_("New password keyring"), "",
+	{ "keyring-new", NULL, N_("New password keyring"), "",
 	  N_("Used to store application and network passwords"), G_CALLBACK (on_new_keyring) },
-	{ "gkr-item-new", NULL, N_("New password..."), "",
+	{ "keyring-item-new", NULL, N_("New password..."), "",
 	  N_("Safely store a password or secret."), G_CALLBACK (on_new_item) },
 };
 
 static const GtkActionEntry ENTRIES_NEW[] = {
-	{ "gkr-add-keyring", "folder", N_("Password Keyring"), "",
+	{ "keyring-new", "folder", N_("Password Keyring"), "",
 	  N_("Used to store application and network passwords"), G_CALLBACK (on_new_keyring) },
-	{ "gkr-add-item", GCR_ICON_PASSWORD, N_("Stored Password"), "",
+	{ "keyring-item-new", GCR_ICON_PASSWORD, N_("Stored Password"), "",
 	  N_("Safely store a password or secret."), G_CALLBACK (on_new_item) }
 };
 
 static const gchar* BACKEND_UI =
 "<ui>"
 "	<popup name='SeahorseGkrBackend'>"
-"		<menuitem action='gkr-keyring-new'/>"
+"		<menuitem action='keyring-new'/>"
 "	</popup>"\
 "</ui>";
 
@@ -101,7 +102,7 @@ seahorse_gkr_backend_actions_init (SeahorseGkrBackendActions *self)
 	GtkActionGroup *actions = GTK_ACTION_GROUP (self);
 
 	gtk_action_group_set_translation_domain (actions, GETTEXT_PACKAGE);
-	gtk_action_group_add_actions (actions, BACKEND_ACTIONS, G_N_ELEMENTS (BACKEND_ACTIONS), self);
+	gtk_action_group_add_actions (actions, BACKEND_ACTIONS, G_N_ELEMENTS (BACKEND_ACTIONS), NULL);
 	seahorse_actions_register_definition (SEAHORSE_ACTIONS (self), BACKEND_UI);
 
 	/* Register another set of actions as a generator */
@@ -125,7 +126,7 @@ seahorse_gkr_backend_actions_instance (void)
 
 	if (actions == NULL) {
 		actions = g_object_new (SEAHORSE_TYPE_GKR_BACKEND_ACTIONS,
-		                        "name", "gkr-backend",
+		                        "name", "KeyringBackend",
 		                        NULL);
 		g_object_add_weak_pointer (G_OBJECT (actions),
 		                           (gpointer *)&actions);
@@ -145,17 +146,13 @@ seahorse_gkr_backend_actions_instance (void)
 
 typedef struct {
 	SeahorseActions parent;
-	GtkAction *action_lock;
-	GtkAction *action_unlock;
-	GtkAction *action_default;
 } SeahorseGkrKeyringActions;
 
 typedef struct {
 	SeahorseActionsClass parent_class;
 } SeahorseGkrKeyringActionsClass;
 
-GType                      seahorse_gkr_keyring_actions_get_type               (void);
-
+GType     seahorse_gkr_keyring_actions_get_type (void) G_GNUC_CONST;
 
 G_DEFINE_TYPE (SeahorseGkrKeyringActions, seahorse_gkr_keyring_actions, SEAHORSE_TYPE_ACTIONS);
 
@@ -176,14 +173,14 @@ on_keyring_unlock_done (GnomeKeyringResult result,
 }
 
 static void
-on_keyring_unlock (GtkAction *action,
-                   gpointer user_data)
+on_keyrings_unlock (GtkAction *action,
+                    gpointer user_data)
 {
-	GList *keys, *l;
+	GList *keys = user_data;
 	GtkWindow *parent;
+	GList *l;
 
 	parent = seahorse_action_get_window (action);
-	keys = seahorse_action_get_objects (action);
 	for (l = keys; l; l = g_list_next (l)) {
 		g_return_if_fail (SEAHORSE_IS_GKR_KEYRING (l->data));
 		gnome_keyring_unlock (seahorse_gkr_keyring_get_name (l->data), NULL,
@@ -191,6 +188,15 @@ on_keyring_unlock (GtkAction *action,
 	}
 }
 
+static void
+on_keyring_unlock (GtkAction *action,
+                   gpointer user_data)
+{
+	GtkWindow *parent = seahorse_action_get_window (action);
+	gnome_keyring_unlock (seahorse_gkr_keyring_get_name (SEAHORSE_GKR_KEYRING (user_data)), NULL,
+	                      on_keyring_unlock_done, g_object_ref (parent), g_object_unref);
+}
+
 static void
 on_keyring_lock_done (GnomeKeyringResult result, gpointer user_data)
 {
@@ -207,14 +213,14 @@ on_keyring_lock_done (GnomeKeyringResult result, gpointer user_data)
 }
 
 static void
-on_keyring_lock (GtkAction *action,
-                 gpointer user_data)
+on_keyrings_lock (GtkAction *action,
+                  gpointer user_data)
 {
-	GList *keyrings, *l;
+	GList *keyrings = user_data;
 	GtkWindow *parent;
+	GList *l;
 
 	parent = seahorse_action_get_window (action);
-	keyrings = seahorse_action_get_objects (action);
 	for (l = keyrings; l; l = g_list_next (l)) {
 		g_return_if_fail (SEAHORSE_IS_GKR_KEYRING (l->data));
 		gnome_keyring_lock (seahorse_gkr_keyring_get_name (l->data),
@@ -222,6 +228,15 @@ on_keyring_lock (GtkAction *action,
 	}
 }
 
+static void
+on_keyring_lock (GtkAction *action,
+                 gpointer user_data)
+{
+	GtkWindow *parent = seahorse_action_get_window (action);
+	gnome_keyring_lock (seahorse_gkr_keyring_get_name (SEAHORSE_GKR_KEYRING (user_data)),
+	                    on_keyring_lock_done, g_object_ref (parent), g_object_unref);
+}
+
 static void
 on_set_default_keyring_done (GnomeKeyringResult result,
                              gpointer user_data)
@@ -242,11 +257,8 @@ static void
 on_keyring_default (GtkAction *action,
                     gpointer user_data)
 {
-	SeahorseGkrKeyring *keyring;
-	GtkWindow *parent;
-
-	parent = seahorse_action_get_window (action);
-	keyring = seahorse_action_get_object (action);
+	SeahorseGkrKeyring *keyring = SEAHORSE_GKR_KEYRING (user_data);
+	GtkWindow *parent = seahorse_action_get_window (action);
 	gnome_keyring_set_default_keyring (seahorse_gkr_keyring_get_name (keyring),
 	                                   on_set_default_keyring_done,
 	                                   g_object_ref (parent), g_object_unref);
@@ -271,24 +283,19 @@ static void
 on_keyring_password (GtkAction *action,
                      gpointer user_data)
 {
-	GtkWindow *window;
-	GList *keys, *l;
+	SeahorseGkrKeyring *keyring = SEAHORSE_GKR_KEYRING (user_data);
+	GtkWindow *window = seahorse_action_get_window (action);
 
-	window = seahorse_action_get_window (action);
-	keys = seahorse_action_get_objects (action);
-	for (l = keys; l; l = g_list_next (l)) {
-		g_return_if_fail (SEAHORSE_IS_GKR_KEYRING (l->data));
-		gnome_keyring_change_password (seahorse_gkr_keyring_get_name (l->data), NULL, NULL,
-		                               on_change_password_done, g_object_ref (window), g_object_unref);
-	}
+	gnome_keyring_change_password (seahorse_gkr_keyring_get_name (keyring),
+	                               NULL, NULL, on_change_password_done,
+	                               g_object_ref (window), g_object_unref);
 }
 
-
 static void
 on_keyring_properties (GtkAction* action,
                        gpointer user_data)
 {
-	seahorse_gkr_keyring_properties_show (seahorse_action_get_object (action),
+	seahorse_gkr_keyring_properties_show (SEAHORSE_GKR_KEYRING (user_data),
 	                                      seahorse_action_get_window (action));
 }
 
@@ -307,16 +314,15 @@ on_delete_objects_complete (GObject *source, GAsyncResult *result, gpointer user
 }
 
 static void
-on_keyring_delete (GtkAction* action,
-                   gpointer user_data)
+on_keyrings_delete (GtkAction* action,
+                    gpointer user_data)
 {
+	GList *objects = user_data;
 	GCancellable *cancellable;
 	GtkWindow *parent;
 	gchar *prompt;
 	gboolean ret;
-	GList *objects;
 
-	objects = seahorse_action_get_objects (action);
 	if (!objects)
 		return;
 
@@ -334,37 +340,45 @@ on_keyring_delete (GtkAction* action,
 		                                               g_list_length (objects)), TRUE);
 		g_object_unref (cancellable);
 	} else {
-		seahorse_action_cancel (action);
+		g_cancellable_cancel (g_cancellable_get_current ());
 	}
 
 	g_free (prompt);
 }
 
+static const GtkActionEntry KEYRINGS_ACTIONS[] = {
+	{ "keyring-lock", NULL, N_("_Lock"), "",
+	  N_("Lock the password storage keyring so a master password is required to unlock it."), G_CALLBACK (on_keyrings_lock) },
+	{ "keyring-unlock", NULL, N_("_Unlock"), "",
+	  N_("Unlock the password storage keyring with a master password so it is available for use."), G_CALLBACK (on_keyrings_unlock) },
+	{ "keyring-delete", GTK_STOCK_DELETE, NULL, NULL,
+	  N_("Delete the keyring."), G_CALLBACK (on_keyrings_delete) },
+};
 
 static const GtkActionEntry KEYRING_ACTIONS[] = {
-	{ "gkr-keyring-lock", NULL, N_("_Lock"), "",
-	  N_("Lock the password storage keyring so a master password is required to unlock it."), G_CALLBACK (on_keyring_lock) },
-	{ "gkr-keyring-unlock", NULL, N_("_Unlock"), "",
-	  N_("Unlock the password storage keyring with a master password so it is available for use."), G_CALLBACK (on_keyring_unlock) },
-	{ "gkr-keyring-default", NULL, N_("_Set as default"), "",
+	{ "keyring-default", NULL, N_("_Set as default"), "",
 	  N_("Applications usually store new passwords in the default keyring."), G_CALLBACK (on_keyring_default) },
-	{ "gkr-keyring-password", NULL, N_("Change _Password"), "",
+	{ "keyring-password", NULL, N_("Change _Password"), "",
 	  N_("Change the unlock password of the password storage keyring"), G_CALLBACK (on_keyring_password) },
-	{ "gkr-keyring-properties", GTK_STOCK_PROPERTIES, NULL, NULL,
+	{ "keyring-properties", GTK_STOCK_PROPERTIES, NULL, NULL,
 	  N_("Properties of the keyring."), G_CALLBACK (on_keyring_properties) },
-	{ "gkr-keyring-delete", GTK_STOCK_DELETE, NULL, NULL,
-	  N_("Delete the keyring."), G_CALLBACK (on_keyring_delete) },
+
+	/* Generic actions used by the sidebar for the current selection */
+	{ "lock", NULL, NULL, "",
+	  N_("Lock the password storage keyring."), G_CALLBACK (on_keyring_lock) },
+	{ "unlock", NULL, NULL, "",
+	  N_("Unlock the password storage keyring."), G_CALLBACK (on_keyring_unlock) },
 };
 
 static const gchar* KEYRING_UI =
 "<ui>"
 "	<popup name='SeahorseGkrKeyring'>"
-"		<menuitem action='gkr-keyring-unlock'/>"
-"		<menuitem action='gkr-keyring-lock'/>"
-"		<menuitem action='gkr-keyring-default'/>"
-"		<menuitem action='gkr-keyring-password'/>"
-"		<menuitem action='gkr-keyring-delete'/>"
-"		<menuitem action='gkr-keyring-properties'/>"
+"		<menuitem action='keyring-unlock'/>"
+"		<menuitem action='keyring-lock'/>"
+"		<menuitem action='keyring-default'/>"
+"		<menuitem action='keyring-password'/>"
+"		<menuitem action='keyring-delete'/>"
+"		<menuitem action='keyring-properties'/>"
 "	</popup>"\
 "</ui>";
 
@@ -372,62 +386,69 @@ static void
 seahorse_gkr_keyring_actions_init (SeahorseGkrKeyringActions *self)
 {
 	GtkActionGroup *actions = GTK_ACTION_GROUP (self);
+
+	/* Add these actions, but none of them are visible until cloned */
 	gtk_action_group_set_translation_domain (actions, GETTEXT_PACKAGE);
-	gtk_action_group_add_actions (actions, KEYRING_ACTIONS, G_N_ELEMENTS (KEYRING_ACTIONS), self);
-	self->action_lock = g_object_ref (gtk_action_group_get_action (actions, "gkr-keyring-lock"));
-	self->action_unlock = g_object_ref (gtk_action_group_get_action (actions, "gkr-keyring-unlock"));
-	self->action_default = g_object_ref (gtk_action_group_get_action (actions, "gkr-keyring-default"));
+	gtk_action_group_add_actions (actions, KEYRINGS_ACTIONS, G_N_ELEMENTS (KEYRINGS_ACTIONS), NULL);
+	gtk_action_group_add_actions (actions, KEYRING_ACTIONS, G_N_ELEMENTS (KEYRING_ACTIONS), NULL);
+	gtk_action_group_set_sensitive (actions, FALSE);
+
 	seahorse_actions_register_definition (SEAHORSE_ACTIONS (self), KEYRING_UI);
 }
 
-static void
-seahorse_gkr_keyring_actions_update (SeahorseActions *actions,
-                                     GList *objects)
+static GtkActionGroup *
+seahorse_gkr_keyring_actions_clone_for_objects (SeahorseActions *actions,
+                                                GList *objects)
 {
-	SeahorseGkrKeyringActions *self = SEAHORSE_GKR_KEYRING_ACTIONS (actions);
-	GnomeKeyringInfo *info;
 	gboolean locked = FALSE;
 	gboolean unlocked = FALSE;
-	gboolean can_default = FALSE;
+	GtkActionGroup *cloned;
 	GList *l;
 
+	g_return_val_if_fail (objects != NULL, NULL);
+
 	for (l = objects; l; l = g_list_next (l)) {
-		info = seahorse_gkr_keyring_get_info (l->data);
-		if (info != NULL) {
-			if (gnome_keyring_info_get_is_locked (info))
-				locked = TRUE;
-			else
-				unlocked = TRUE;
-			if (!seahorse_gkr_keyring_get_is_default (l->data))
-				can_default = TRUE;
-		}
+		if (seahorse_gkr_keyring_get_locked (l->data))
+			locked = TRUE;
+		else
+			unlocked = TRUE;
 	}
 
-	gtk_action_set_sensitive (self->action_lock, unlocked);
-	gtk_action_set_sensitive (self->action_unlock, locked);
-	gtk_action_set_sensitive (self->action_default, can_default);
-}
+	cloned = gtk_action_group_new ("KeyringObject");
+	gtk_action_group_set_translation_domain (cloned, GETTEXT_PACKAGE);
 
-static void
-seahorse_gkr_keyring_actions_finalize (GObject *obj)
-{
-	SeahorseGkrKeyringActions *self = SEAHORSE_GKR_KEYRING_ACTIONS (obj);
+	gtk_action_group_add_actions_full (cloned, KEYRINGS_ACTIONS,
+	                                   G_N_ELEMENTS (KEYRINGS_ACTIONS),
+	                                   seahorse_object_list_copy (objects),
+	                                   seahorse_object_list_free);
+	gtk_action_set_sensitive (gtk_action_group_get_action (cloned, "keyring-lock"), unlocked);
+	gtk_action_set_sensitive (gtk_action_group_get_action (cloned, "keyring-unlock"), locked);
 
-	g_clear_object (&self->action_lock);
-	g_clear_object (&self->action_unlock);
-	g_clear_object (&self->action_default);
+	/* Single object */
+	if (!objects->next) {
+		gtk_action_group_add_actions_full (cloned, KEYRING_ACTIONS,
+		                                   G_N_ELEMENTS (KEYRING_ACTIONS),
+		                                   g_object_ref (objects->data),
+		                                   g_object_unref);
+		g_object_bind_property (objects->data, "is-default",
+		                        gtk_action_group_get_action (cloned, "keyring-default"), "sensitive",
+		                        G_BINDING_INVERT_BOOLEAN | G_BINDING_SYNC_CREATE);
+		g_object_bind_property (objects->data, "locked",
+		                        gtk_action_group_get_action (cloned, "lock"), "sensitive",
+		                        G_BINDING_INVERT_BOOLEAN | G_BINDING_SYNC_CREATE);
+		g_object_bind_property (objects->data, "locked",
+		                        gtk_action_group_get_action (cloned, "unlock"), "sensitive",
+		                        G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE);
+	}
 
-	G_OBJECT_CLASS (seahorse_gkr_keyring_actions_parent_class)->finalize (obj);
+	return cloned;
 }
 
 static void
 seahorse_gkr_keyring_actions_class_init (SeahorseGkrKeyringActionsClass *klass)
 {
 	SeahorseActionsClass *actions_class = SEAHORSE_ACTIONS_CLASS (klass);
-	GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
-
-	gobject_class->finalize = seahorse_gkr_keyring_actions_finalize;
-	actions_class->update = seahorse_gkr_keyring_actions_update;
+	actions_class->clone_for_objects = seahorse_gkr_keyring_actions_clone_for_objects;
 }
 
 GtkActionGroup *
@@ -470,7 +491,7 @@ static void
 on_password_properties (GtkAction *action,
                         gpointer user_data)
 {
-	seahorse_gkr_item_properties_show (seahorse_action_get_object (action),
+	seahorse_gkr_item_properties_show (SEAHORSE_GKR_ITEM (user_data),
 	                                   seahorse_action_get_window (action));
 }
 
@@ -501,7 +522,7 @@ on_delete_passwords (GtkAction *action,
 	gboolean ret;
 	guint num;
 
-	objects = seahorse_action_get_objects (action);
+	objects = user_data;
 	if (objects == NULL)
 		return;
 
@@ -524,7 +545,7 @@ on_delete_passwords (GtkAction *action,
 		seahorse_progress_show (cancellable, ngettext ("Deleting item", "Deleting items", num), TRUE);
 		g_object_unref (cancellable);
 	} else {
-		seahorse_action_cancel (action);
+		g_cancellable_cancel (g_cancellable_get_current ());
 
 	}
 
@@ -534,6 +555,9 @@ on_delete_passwords (GtkAction *action,
 static const GtkActionEntry ITEM_ACTIONS[] = {
 	{ "properties", GTK_STOCK_PROPERTIES, NULL, NULL,
 	  N_("Properties of the password."), G_CALLBACK (on_password_properties) },
+};
+
+static const GtkActionEntry ITEMS_ACTIONS[] = {
 	{ "delete", GTK_STOCK_DELETE, NULL, NULL,
 	  N_("Delete the password."), G_CALLBACK (on_delete_passwords) },
 };
@@ -541,15 +565,36 @@ static const GtkActionEntry ITEM_ACTIONS[] = {
 static void
 seahorse_gkr_item_actions_init (SeahorseGkrItemActions *self)
 {
-	GtkActionGroup *actions = GTK_ACTION_GROUP (self);
-	gtk_action_group_set_translation_domain (actions, GETTEXT_PACKAGE);
-	gtk_action_group_add_actions (actions, ITEM_ACTIONS, G_N_ELEMENTS (ITEM_ACTIONS), self);
+
+}
+
+static GtkActionGroup *
+seahorse_gkr_item_actions_clone_for_objects (SeahorseActions *actions,
+                                             GList *objects)
+{
+	GtkActionGroup *cloned;
+
+	cloned = gtk_action_group_new ("KeyringItem");
+	gtk_action_group_set_translation_domain (cloned, GETTEXT_PACKAGE);
+	gtk_action_group_add_actions_full (cloned, ITEMS_ACTIONS,
+	                                   G_N_ELEMENTS (ITEMS_ACTIONS),
+	                                   seahorse_object_list_copy (objects),
+	                                   seahorse_object_list_free);
+
+	if (!objects->next)
+		gtk_action_group_add_actions_full (cloned, ITEM_ACTIONS,
+		                                   G_N_ELEMENTS (ITEM_ACTIONS),
+		                                   g_object_ref (objects->data),
+		                                   g_object_unref);
+
+	return cloned;
 }
 
 static void
 seahorse_gkr_item_actions_class_init (SeahorseGkrItemActionsClass *klass)
 {
-
+	SeahorseActionsClass *actions_class = SEAHORSE_ACTIONS_CLASS (klass);
+	actions_class->clone_for_objects = seahorse_gkr_item_actions_clone_for_objects;
 }
 
 GtkActionGroup *
@@ -559,7 +604,7 @@ seahorse_gkr_item_actions_instance (void)
 
 	if (actions == NULL) {
 		actions = g_object_new (SEAHORSE_TYPE_GKR_ITEM_ACTIONS,
-		                        "name", "gkr-item",
+		                        "name", "KeyringItem",
 		                        NULL);
 		g_object_add_weak_pointer (G_OBJECT (actions),
 		                           (gpointer *)&actions);
