@@ -1177,12 +1177,15 @@ on_pgp_details_export_button (GtkWidget *widget,
 	uri = seahorse_util_chooser_save_prompt (dialog);
 	if (!uri) 
 		return;
-	
+
+	ctx = seahorse_gpgme_keyring_new_context (&gerr);
+	if (ctx == NULL)
+		return;
+
 	/* Export to a data block */
 	gerr = gpgme_data_new (&data);
 	g_return_if_fail (GPG_IS_OK (gerr));
-	ctx = seahorse_gpgme_keyring_new_context ();
-	g_return_if_fail (ctx);
+
 	gerr = seahorse_gpg_op_export_secret (ctx, seckey->subkeys->keyid, data);
 	gpgme_release (ctx);
 	results = gpgme_data_release_and_get_mem (data, &n_results);
@@ -1876,9 +1879,9 @@ do_trust (SeahorseWidget *swidget)
 			g_signal_connect (widget, "toggled", G_CALLBACK (on_pgp_trusted_toggled), filter);
 			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
 		}
-	}
 
-	signatures_populate_model (swidget, SEAHORSE_OBJECT_MODEL (store));
+		signatures_populate_model (swidget, SEAHORSE_OBJECT_MODEL (store));
+	}
 }
 
 /* -----------------------------------------------------------------------------
@@ -1981,8 +1984,10 @@ seahorse_pgp_key_properties_show (SeahorsePgpKey *pkey, GtkWindow *parent)
 	SeahorseWidget *swidget;
 
 	/* This causes the key source to get any specific info about the key */
-	if (SEAHORSE_IS_GPGME_KEY (pkey))
+	if (SEAHORSE_IS_GPGME_KEY (pkey)) {
 		seahorse_gpgme_key_refresh (SEAHORSE_GPGME_KEY (pkey));
+		seahorse_gpgme_key_ensure_signatures (SEAHORSE_GPGME_KEY (pkey));
+	}
 
 	if (seahorse_object_get_usage (SEAHORSE_OBJECT (pkey)) == SEAHORSE_USAGE_PUBLIC_KEY)
 		swidget = setup_public_properties (pkey, parent);
