@@ -2,6 +2,7 @@
  * Seahorse
  *
  * Copyright (C) 2005 Stefan Walter
+ * Copyright (C) 2011 Collabora Ltd.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,7 +40,6 @@
 #define __SEAHORSE_SSH_SOURCE_H__
 
 #include "seahorse-ssh-key.h"
-#include "seahorse-source.h"
 
 #define SEAHORSE_TYPE_SSH_SOURCE            (seahorse_ssh_source_get_type ())
 #define SEAHORSE_SSH_SOURCE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), SEAHORSE_TYPE_SSH_SOURCE, SeahorseSSHSource))
@@ -66,8 +66,8 @@ GType                seahorse_ssh_source_get_type           (void);
 
 SeahorseSSHSource*   seahorse_ssh_source_new                (void);
 
-struct _SeahorseSSHKey*      
-                     seahorse_ssh_source_key_for_filename   (SeahorseSSHSource *ssrc, 
+struct _SeahorseSSHKey*
+                     seahorse_ssh_source_key_for_filename   (SeahorseSSHSource *ssrc,
                                                              const gchar *privfile);
 
 gchar*               seahorse_ssh_source_file_for_public    (SeahorseSSHSource *ssrc,
@@ -76,10 +76,21 @@ gchar*               seahorse_ssh_source_file_for_public    (SeahorseSSHSource *
 gchar*               seahorse_ssh_source_file_for_algorithm (SeahorseSSHSource *ssrc,
                                                              guint algo);
 
+void                 seahorse_ssh_source_remove_object      (SeahorseSSHSource *self,
+                                                             SeahorseSSHKey *skey);
+
 guchar*              seahorse_ssh_source_export_private     (SeahorseSSHSource *ssrc,
                                                              SeahorseSSHKey *skey,
                                                              gsize *n_results,
                                                              GError **err);
 
+void                 seahorse_ssh_source_load_async         (SeahorseSSHSource *self,
+                                                             GCancellable *cancellable,
+                                                             GAsyncReadyCallback callback,
+                                                             gpointer user_data);
+
+gboolean             seahorse_ssh_source_load_finish        (SeahorseSSHSource *self,
+                                                             GAsyncResult *result,
+                                                             GError **error);
 
 #endif /* __SEAHORSE_SSH_SOURCE_H__ */
